function varargout=wcsimconmask(Action,varargin)
%WCSIMCONMASK  Callback function for Worst-case Simulation Constraint block.
%
%This function is meant to be called only by the WCSIMCON Block mask.
%
%See also RCTBLOCKS, CSTBLOCKS, LTIMASK, USS, UMAT, UREAL, ULTIDYN.


lwarn = lastwarn;warn = warning('off');

switch Action,
    case 'Initialize'
        % Set variable name in underlying ToWorkspace block
        blk = varargin{1};
        blkh = get_param(blk,'Handle');
        VarName = get(blkh,'VarName');
        %b=find_system(blk,'LookUnderMasks','on','FollowLinks','on','BlockType','ToWorkspace')        
        %bh = get_param(b{1},'Handle');
        bh = get_param([blk '/To Workspace'],'Handle');
        set(bh,'VariableName',VarName);
    case 'Stop',
        % Compute constraint function
        blk = varargin{1};
        blkh = get_param(blk,'Handle');
        VarName = get(blkh,'VarName');
        ConType = get(blkh,'ConType');
        ConUB = evalin('base',get(blkh,'ConUB'));
        ConLB = evalin('base',get(blkh,'ConLB'));
        %VarNameVal = evalin('caller',VarName);        
        try
            VarNameVal = evalin('base',VarName);                
        catch
            % Simulation failed to run
            return;
        end
        t = VarNameVal.time;               % Nt x 1
        y = VarNameVal.signals.values;     % Nt x Ny
        switch ConType
            case 'L1 Norm'
                con = trapz(t,sum(abs(y),2) );
            case 'L2 Norm'
                con = trapz(t, sum(y.^2,2) ); 
                con = sqrt(con);
            case 'Linf Norm'
                con = max(max(abs(y)));
            case 'Pointwise Maximum'
                con = max(y(:));
            case 'Pointwise Minimum'
                con = min(y(:));
            case 'M-file'
                % M-file should be of the form con = f(t,y)
                ConMfile = get(blkh,'ConMfile');
                %con = eval([ConMfile '(t,y)']);
                con = feval(ConMfile,t,y);
            otherwise
                error('Invalid Constraint Function Type')
        end        
        VarNameVal.constraint.value = con;
        VarNameVal.constraint.upperbound = ConUB;
        VarNameVal.constraint.lowerbound = ConLB;
        VarNameVal.blockName = blk;
        assignin('base',VarName,VarNameVal);
    case 'MaskCallback'
        blk = varargin{1};
        blkh = get_param(blk,'Handle');
        VarName = get(blkh,'VarName');
        ConType = get(blkh,'ConType');
        if strcmp(ConType,'M-file')            
            set(blkh,'MaskEnables',{'on','on','on','on','on','on','on','on','off','off'});
        else
            set(blkh,'MaskEnables',{'on','off','on','on','on','on','on','on','off','off'});
        end                      
    case 'Copy',
        % Mimic variable naming convention used by ToWorkspace blocks
        [blk,mdl]=deal(varargin{:});
        %mdl = gcs;                                  
        b = find_system(mdl,'Tag','WCSIMConstraint');
        b = setdiff(b,blk);                         
        VN = get_param(b,'VarName');                 
        n=1;                                        
        go = 1;                                     
        while go && (n<1e5)                          
            str = 'WCSIMConstraint';                  
            if n~=1                                  
                str = [str int2str(n)];                
            end                                      
            if any(strcmp(str,VN))                    
                n=n+1;                                
            else                                     
                go=0;                                  
            end                                      
        end                                         
        set_param(blk,'VarName',str);               
    % case 'Update Diagram'
    %   XXX Do we need an 'Update Diagram' like ussmask?
end       
              
warning(warn);lastwarn(lwarn);
