function varargout=wcsimobjmask(Action,varargin)
%WCSIMOBJMASK  Callback function for Worst-case Simulation Objective block.
%
%This function is meant to be called only by the WCSIMOBJ Block mask.
%
%See also RCTBLOCKS, CSTBLOCKS, LTIMASK, USS, UMAT, UREAL, ULTIDYN.


lwarn = lastwarn;warn = warning('off');

switch Action,
    case 'Initialize'
        % Set variable name in underlying ToWorkspace block
        blk = varargin{1};
        blkh = get_param(blk,'Handle');
        VarName = get(blkh,'VarName');
        %b=find_system(blk,'LookUnderMasks','on','FollowLinks','on','BlockType','ToWorkspace')        
        %bh = get_param(b{1},'Handle');
        bh = get_param([blk '/To Workspace'],'Handle');
        set(bh,'VariableName',VarName);
    case 'Stop',
        % Compute objective function
        blk = varargin{1};
        blkh = get_param(blk,'Handle');
        VarName = get(blkh,'VarName');
        ObjType = get(blkh,'ObjType');
        try
            VarNameVal = evalin('base',VarName);        
        catch
            % Simulation failed to run
            return;
        end
        t = VarNameVal.time;               % Nt x 1
        y = VarNameVal.signals.values;     % Nt x Ny
        switch ObjType
            case 'L1 Norm'
                obj = trapz(t,sum(abs(y),2) );
            case 'L2 Norm'
                obj = trapz(t, sum(y.^2,2) ); 
                obj = sqrt(obj);
            case 'Linf Norm'
                obj = max(max(abs(y)));
            case 'Pointwise Maximum'
                obj = max(y(:));
            case 'Pointwise Minimum'
                obj = min(y(:));
            case 'M-file'
                % M-file should be of the form obj = f(t,y)
                ObjMfile = get(blkh,'ObjMfile');
                obj = eval([ObjMfile '(t,y)']);
            otherwise
                error('Invalid Objective Function Type')
        end
        VarNameVal.objective.value = obj;
        VarNameVal.blockName = blk;
        assignin('base',VarName,VarNameVal);
    case 'MaskCallback'
        blk = varargin{1};
        blkh = get_param(blk,'Handle');
        VarName = get(blkh,'VarName');
        ObjType = get(blkh,'ObjType');
        if strcmp(ObjType,'M-file')
            set(blkh,'MaskEnables',{'on','on','on','on','on','on','on','off','off'});
        else
            set(blkh,'MaskEnables',{'on','on','off','on','on','on','on','off','off'});
        end                      
    case 'Copy',
        % Mimic variable naming convention used by ToWorkspace blocks
        [blk,mdl]=deal(varargin{:});
        %mdl = gcs;                                  
        b = find_system(mdl,'Tag','WCSIMObjective');
        b = setdiff(b,blk);                         
        VN = get_param(b,'VarName');                 
        n=1;                                        
        go = 1;                                     
        while go && (n<100)                          
            str = 'WCSIMObjective';                  
            if n~=1                                  
                str = [str int2str(n)];                
            end                                      
            if any(strcmp(str,VN))                    
                n=n+1;                                
            else                                     
                go=0;                                  
            end                                      
        end                                         
        set_param(blk,'VarName',str);               
    % case 'Update Diagram'
    %   XXX Do we need an 'Update Diagram' like ussmask?
end       
       
       
warning(warn);lastwarn(lwarn);
